%% Algoritmo de abejas para resolver de manera general probles de optmizacion
% Este programa resuleve la funcion EggHolder.
% Este programa resulve el problema de clustering.

clc, clear, close all;

%% Parametros iniciales 
D     =    2;           % 2 Dimensiones (features) para funcion Rosenbrock
Np    =   50;           % Tamano de la poblacion de abejas
Fc    = round(Np/2);    % Fuente de comida
Gmax  =  500;           % Generaciones (Iteraciones) maximas
Limit =   50;           % Criterio de abandono
xmin  = -512;           % Liminte inferior
xmax  =  512;           % Limite superior

%% Poblacion inicial aleatoria
tic
for epocas = 1:1

% Tabla Vacia de abejas 
Abeja_vacia.Posicion       = [];
Abeja_vacia.Costo          = [];
Abeja_vacia.Fitness        = [];
Abeja_vacia.Limite         = [];
Abeja_vacia.Probabi        = [];
Abeja_vacia.Mejor.Posicion = [];
Abeja_vacia.Mejor.Costo    = [];

% Creamos el arreglo de la poblacion de abejas las Abejas
Abejas = repmat(Abeja_vacia, Np, 1);

% Se guarda en memoria el mejor dato global de las abejas
Mejor_Global.Costo = inf;
Mejor_Global.Iteracion = 0;

i = 1;
while (i <= Np)
    Abejas(i).Posicion = unifrnd(xmin, xmax, [1, D]);
    Abejas(i).Costo    = eggholder( Abejas(i).Posicion );
    Abejas(i).Fitness  = Fitness(Abejas(i).Costo);
    Abejas(i).Limite   = 0;
    Abejas(i).Probabi  = 0;
    
    % Actualizar el mejor valor global
    if Abejas(i).Costo < Mejor_Global.Costo
        Mejor_Global = Abejas(i);
    end
    
    i = i + 1;
    
end

%% Inicia el bucle de las abejas

g = 1;                  % Contador de generaciones (iteraciones)
while ( g <= Gmax )
    
    %====================== Incio de face abejas obreras =================%
    a = 1;              % numero de abeja
    while (a <= Np)
        
        % Parametro de dimension a modificar aleatoriamente (cualquier feature)
        DimChange = fix( rand*D ) + 1;
        
        % Compara abeja actual con otra del enjambre
        Vecino = fix( rand*Np ) + 1;
        while( Vecino == a )
            Vecino = fix( rand*Np ) + 1;
        end
        
        V   = Abejas(a);
        phi = (rand - 0.5) * 2;
        V.Posicion(1, DimChange) = Abejas(a).Posicion(1, DimChange) + ...
               ( phi * (Abejas(a).Posicion(1, DimChange) - Abejas(Vecino).Posicion(1, DimChange)));
        
        %Si el valor del nuevo parametro esta fuera de los limites hay que restringirlo
        ind = find( V.Posicion < xmin );
        V.Posicion(1, ind) = xmin;
        ind = find( V.Posicion > xmax );
        
        V.Posicion(1, ind) = xmax;
        V.Costo            = eggholder( V.Posicion );
        V.Fitness          = Fitness(V.Costo);
        V.Limite           = 0;
        
        % Se toma el mejor valor
        if V.Costo < Abejas(a).Costo
            Abejas(a) = V;
        else
            Abejas(a).Limite = Abejas(a).Limite + 1;
        end
        
        % Actualizar el mejor valor global
        if Abejas(a).Costo < Mejor_Global.Costo
            Mejor_Global = Abejas(a);
            Mejor_Global.Iteracion = g;
        end
        
        a = a + 1;
    end
    %=====================================================================%
    %================= Incio de face abejas observadoras =================%
    % Se calcula la probabilidad utilizando los valores de fitness
    % normalizados
    i = 1;
    while (i <= Np)
        Abejas(i).Probabi = ( 0.9 * (Abejas(i).Fitness/Mejor_Global.Fitness) ) + 0.1;
        i = i + 1;
    end
    
    i=1;
    t=0;
    while(t <= Np)
        if( rand <= Abejas(i).Probabi )
            t = t+1;
            
            % Parametro de dimension a modificar aleatoriamente (cualquier feature)
            DimChange = fix( rand*D ) + 1;
            
            % Compara abeja actual con otra del enjambre
            Vecino = fix( rand*Np ) + 1;
            while( Vecino == i )
                Vecino = fix( rand*Np ) + 1;
            end
            
            V   = Abejas(i);
            phi = (rand - 0.5) * 2;
            V.Posicion(1, DimChange) = Abejas(i).Posicion(1, DimChange) + ...
                ( phi * (Abejas(i).Posicion(1, DimChange) - Abejas(Vecino).Posicion(1, DimChange)));
            
            %Si el valor del nuevo parametro esta fuera de los limites hay que restringirlo
            ind = find( V.Posicion < xmin );
            V.Posicion(1, ind) = xmin;
            ind = find( V.Posicion > xmax );
            
            V.Posicion(1, ind) = xmax;
            V.Costo            = eggholder( V.Posicion );
            V.Fitness          = Fitness(V.Costo);
            V.Limite           = 0;
            
            % Se toma el mejor valor
            if V.Costo < Abejas(i).Costo
                Abejas(i) = V;
            else
                Abejas(i).Limite = Abejas(i).Limite + 1;
            end
            
            % Actualizar el mejor valor global
            if Abejas(i).Costo < Mejor_Global.Costo
                Mejor_Global = Abejas(i);
                Mejor_Global.Iteracion = g;
            end
            
        end
        i = i+1;
        if (i == (Np)+1)
            i=1;
        end
    end
    %=====================================================================%
    %================== Inicio de face abeja exploradora ==================%
    b = 1;
    while (b <= Np)
        
        if (Abejas(b).Limite >= Limit )
            Abejas(b).Posicion = unifrnd(xmin, xmax, [1, D]);
            Abejas(b).Costo    = eggholder( Abejas(b).Posicion );
            Abejas(b).Fitness  = Fitness(Abejas(b).Costo);
            Abejas(b).Limite   = 0;
            Abejas(b).Probabi  = 0;
            
            % Actualizar el mejor valor global
            if Abejas(b).Costo < Mejor_Global.Costo
                Mejor_Global = Abejas(b);
                Mejor_Global.Iteracion = g;
            end
            
        end
        b = b + 1;
    end
    %=====================================================================%
    % Imprimir informacion
    disp( ['En la iteracion ', num2str(g), ' el valor global del costo es ', num2str(Mejor_Global.Costo)] )
    BestCosto(g) = Mejor_Global.Costo;
    g = g + 1;
    
end

Mejor_Total(epocas) = Mejor_Global;
end
toc

figure;
%plot(BestCost,'LineWidth',2);
semilogy(BestCosto,'LineWidth',2);
xlabel('Iteracin');
ylabel('Valor de Costo de la Funcin');
set(gca,'FontSize',12)
grid on;

%% Esta funcion permite calcular el fitness relativo
function Out = Fitness(In)
    if In > 0
        Out = 1 / (1 + In);
    else
        Out = 1 + abs( In );
    end
end


%% Funcin de optimizacin Eggholder
function out = eggholder(x)
a   = abs( sqrt( x(2) + (x(1)/2) + 47 ) );
b   = abs( sqrt( x(1) - (x(2) + 47) ) );
out = ( -(x(2) + 47)*sin( a ) ) - (x(1)*sin(b) );
end

