%% Algoritmo de hormigas para dominios continuos ACO-R
% Este programa resuleve la funcion EggHolder.
% Este programa resulve el problema de clustering.
clc, clear, close all;

%% Parametros iniciales 
Dim   =    2;       % 2 Dimensiones (features) para funcin Rosenbrock
Xmin  = -512;       % Limite inferior
Xmax  =  512;       % Limite superior
MaxIt =  500;       % Nmero de iteraciones mximas
Nh    =   50;       % Tamao de la poblacin o nmero de hormigas

% Parmetros de la ecuacin de ACO-R
nSample =  40;      % Sample Size
q       = 0.5;      % Intensification Factor (Selection Pressure)
zeta    =   1;      % Deviation-Distance Ratio

%% Poblacion Inicial Aleatoria de Hormigas
tic
for epocas = 1:1

% Se crea el objeto vacio
hormiga_vacia.Posicion = [];
hormiga_vacia.Costo    = [];

% Creamos el arreglo de la poblacin de partculas
hormiga = repmat( hormiga_vacia, Nh, 1 );

% Se guarda en memoria el mejor dato global de las partculas
Mejor_Global.Costo = inf;
Mejor_Global.Iteracion = 0;

i = 1;
while (i <= Nh)
    hormiga(i).Posicion = unifrnd( Xmin, Xmax, [1, Dim] );
    hormiga(i).Costo    = eggholder( hormiga(i).Posicion );
    i = i + 1;
end

% Se arregla del mayor al menor a las hormigas
[~, Ordenado] = sort( [hormiga.Costo] );
hormiga       = hormiga( Ordenado );
BestSolucion  = hormiga(1);     % Se toma el valor mayor
BestCosto     = zeros(MaxIt,1); % Vector para almacenar los mejores valores

% Se calcula el peso de cada solucin
w = 1 / ( sqrt(2*pi)*q*Nh)*exp(-0.5*(((1:Nh)-1)/(q*Nh)).^2 );

% Probabilidad de cada solucin
p = w / sum(w);

%% Ciclo principal del algoritmo ACO-R
g = 1;          % Contador de generaciones (iteraciones)
while ( g <= MaxIt )
    
    % Se calcula el promedio
    s = zeros(Nh,Dim);
    for l = 1:Nh
        s(l,:) = hormiga(l).Posicion;
    end
    
    % Desviacion Estandar
    sigma = zeros( Nh,Dim );
    for l = 1:Nh
        Dd = 0;
        for r = 1:Nh
            Dd = Dd + abs( s(l,:)-s(r,:) );
        end
        sigma(l,:) = zeta*Dd / ( Nh-1 );
    end
    
    % Se crea la posible nueva solucin
    newpop = repmat( hormiga_vacia, nSample, 1 );
    
    a = 1;
    while (a <= nSample)        
        % Initialize Position Matrix
        newpop(a).Posicion = zeros( [1, Dim] );
        
        % Solution Construction
        for i = 1:Dim
            % Seleccin de la campana de Gauss a modificar
            l = RouletteWheelSelection( p );
            
            % Se genera de forma aleatoria una nueva campana de Gauss
            newpop(a).Posicion(i) = s(l,i) + sigma(l,i)*randn;
            
            % Aplicar los limites a la nueva posicin
            newpop(a).Posicion(i) = max( newpop(a).Posicion(i), Xmin);
            newpop(a).Posicion(i) = min( newpop(a).Posicion(i), Xmax);
            
        end
        
        % Evaluar las nuevas posiciones
        newpop(a).Costo = eggholder( newpop(a).Posicion );
        a = a + 1;
    end
    
    % Merge Main Population (Archive) and New Population (Samples)
    hormiga = [hormiga 
         newpop]; % Recordar que esta en forma de concatenacin
     
    % Se arregla del mayor al menor a las hormigas
    [~, Ordenado] = sort( [hormiga.Costo] );
    hormiga       = hormiga( Ordenado );
    
    % Se borran las posiciones extras generadas
    hormiga = hormiga( 1:Nh );
    
    % Se actualiza la mejor solucion del vector
    BestSolucion = hormiga(1);
    BestCosto(g) = hormiga(1).Costo;
    
    % Actualizar el mejor Global
    if ( hormiga(1).Costo < Mejor_Global.Costo )
        Mejor_Global = hormiga(1);
        Mejor_Global.Iteracion = g;
    end
    
    disp(['Iteracin ' num2str(g) ': Best Costo = ' num2str( Mejor_Global.Costo )]);
    g = g + 1;
end

Mejor_Total(epocas) = Mejor_Global;
end
toc

%% Grfica de desempeo
figure;
%plot(BestCost,'LineWidth',2);
semilogy(BestCosto,'LineWidth',2);
xlabel('Iteracin');
ylabel('Valor de Costo de la Funcin');
set(gca,'FontSize',12)
grid on;


%% Funcin de optimizacin Eggholder
function out = eggholder(x)
a   = abs( sqrt( x(2) + (x(1)/2) + 47 ) );
b   = abs( sqrt( x(1) - (x(2) + 47) ) );
out = ( -(x(2) + 47)*sin( a ) ) - (x(1)*sin(b) );
end



