%% Algoritmo de enjambre de particulas resolver de manera general probles de optmizacion
% Este programa resuleve la funcion EggHolder.
% Este programa resulve el problema de clustering.

clc, clear, close all;

%% Parametros iniciales 
D     =    2;       % 2 Dimensiones (features) para funcin Rosenbrock
Xmin  = -512;       % Limite inferior
Xmax  =  512;       % Limite superior
MaxIt =  500;       % Generaciones (Iteraciones) mximas
Np    =   50;       % Tamao de la poblacin o nmero de partculas

% Parmetros de la ecuacin PSO
w     = 1;          % Parametro de Inercia
wdamp = 0.99;       % Diesmado del paramtro de Inercia
c1    = 1.95;       % Coeficiente Personal
c2    = 1.95;       % Coeficiente Global

% % % VelMax = 0.1 * ( Xmax - Xmin );
% % % VelMin = -VelMax;

%% Poblacion de particulas inicial aleatoria
tic
for epocas = 1:1
particula_vacia.Posicion      = [];
particula_vacia.Costo         = [];
particula_vacia.Velocidad     = [];
particula_vacia.Best.Posicion = [];
particula_vacia.Best.Costo    = [];

% Creamos el arreglo de la poblacin de partculas
particula = repmat( particula_vacia, Np, 1 );

% Se guarda en memoria el mejor dato global de las partculas
Mejor_Global.Costo = inf;
Mejor_Global.Iteracion = 0;

i = 1;
while (i <= Np)
    particula(i).Posicion      = unifrnd( Xmin, Xmax, [1, D] );
    particula(i).Velocidad     = zeros( [1, D] );
    particula(i).Costo         = eggholder( particula(i).Posicion );
    particula(i).Best.Posicion = particula(i).Posicion;
    particula(i).Best.Costo    = particula(i).Costo;
    
    % Actualizar el mejor Global
    if particula(i).Best.Costo < Mejor_Global.Costo
        Mejor_Global = particula(i).Best;
    end
    
    i = i + 1;
end

% Vector con todos los mejores Globales
BestCosto = zeros( MaxIt,1 );

%% Ciclo principal del algoritmo PSO

g = 1;          % Contador de generaciones (iteraciones)
while ( g <= MaxIt )
    a = 1;
    while (a <= Np)
        % Actualizar Velocidad
        particula(a).Velocidad = w*particula(a).Velocidad ...
            +c1*rand( [1, D] ).*( particula(a).Best.Posicion - particula(a).Posicion ) ...
            +c2*rand( [1, D] ).*( Mejor_Global.Posicion - particula(a).Posicion );
        
        % Aplicar los limites a la nueva velocidad
% % %         particula(a).Velocidad = max( particula(a).Velocidad, VelMin);
% % %         particula(a).Velocidad = min( particula(a).Velocidad, VelMax);
        
        % Actualizar Posicin
        particula(a).Posicion = particula(a).Posicion + particula(a).Velocidad;
        
        % Aplicar los limites a la nueva posicin
        particula(a).Posicion = max( particula(a).Posicion, Xmin);
        particula(a).Posicion = min( particula(a).Posicion, Xmax);
        
        % Evaluacin de la nueva posicin
        particula(a).Costo = eggholder( particula(a).Posicion );
        
        % Recordar el mejor personal
        if ( particula(a).Costo < particula(a).Best.Costo )
            particula(a).Best.Posicion = particula(a).Posicion;
            particula(a).Best.Costo    = particula(a).Costo;
            
            % Actualizar el mejor Global
            if ( particula(a).Best.Costo < Mejor_Global.Costo )
                Mejor_Global = particula(a).Best;
                Mejor_Global.Iteracion = g;
            end
            
        end
        a = a + 1;
    end
    
    BestCosto(g) = Mejor_Global.Costo;
    disp(['Iteration ' num2str(g) ': Mejor Costo = ' num2str(BestCosto(g))]);
    
    % Se actualiza el coeficiente de Inercia
    w = w*wdamp;

    g = g + 1;
end

Mejor_Total(epocas) = Mejor_Global;
end
toc

%% Grfica de desempeo
figure;
%plot(BestCost,'LineWidth',2);
semilogy(BestCosto,'LineWidth',2);
xlabel('Iteracin');
ylabel('Valor de Costo de la Funcin');
set(gca,'FontSize',12)
grid on;

%% Funcin de optimizacin Eggholder
function out = eggholder(x)
a   = abs( sqrt( x(2) + (x(1)/2) + 47 ) );
b   = abs( sqrt( x(1) - (x(2) + 47) ) );
out = ( -(x(2) + 47)*sin( a ) ) - (x(1)*sin(b) );
end





